import unittest
from src.lenden.dataexport import *



class TestDataExports(unittest.TestCase):

    def get_DataResults1(self):
        desc = "Irure aute ex irure fugiat."
        help = "Ex consequat nostrud enim nulla dolore incididunt officia ullamco ut consequat mollit nulla."
        type = ResultType.image
        path = "/path/to/image"
        size = ResultSizeType.normal
        format = "f2%"
        dataR1 = DataResults(Description=desc, Help=help,
                             Type=type, Path=path, Size=size, Format=format)
        return dataR1

    def get_DataResults2(self):
        desc = "Dolore culpa duis duis nulla duis eiusmod ullamco adipisicing aliqua."
        help = "Dolore mollit nulla.Nulla amet quis ad deserunt sit laboris irure incididunt est ea occaecat sunt."
        type = ResultType.numberarray
        path = "/path/to/numbersfile"
        size = ResultSizeType.big
        format = "%f12%"
        data1 = DataResults(Description=desc, Help=help,
                            Type=type, Path=path, Size=size, Format=format)
        return data1

    def test_DataExport_Not_Equals(self):
        data1 = self.get_DataResults1()
        data2 = self.get_DataResults2()

        data = DataExport(Version=0.1, HasError=False,
                          DisplayType=DisplayType.grid, Results=[data1, data2], Logs=None)

        dem0 = data.getJSON()
        data2 = DataExport.fromJSON(dem0)

        self.assertEqual(data, data2)

    def test_DataExport_With_Error(self):
        data = DataExport(Version=0.1, HasError=True,
                          DisplayType=DisplayType.grid, Results=[], 
                          Logs=DataLog("Voluptate sint sint eu aliqua consequat culpa nisi fugiat nostrud aute adipisicing.", ["/path/log1","/path/log2","/path/log3"]))

        dem0 = data.getJSON()
        data2 = DataExport.fromJSON(dem0)

        self.assertEqual(data, data2)


import unittest
from unittest.mock import patch
from src.lenden.dataexport import *
import json


class TestDataResults(unittest.TestCase):

    def get_DataResults1(self):
        desc = "Irure aute ex irure fugiat."
        help = "Ex consequat nostrud enim nulla dolore incididunt officia ullamco ut consequat mollit nulla."
        type = ResultType.image
        path = "/path/to/image"
        size = ResultSizeType.normal
        format = "f2%"
        dataR1 = DataResults(Description=desc, Help=help,
                             Type=type, Path=path, Size=size, Format=format)
        return dataR1

    def get_DataResults2(self):
        desc = "Dolore culpa duis duis nulla duis eiusmod ullamco adipisicing aliqua."
        help = "Dolore mollit nulla.Nulla amet quis ad deserunt sit laboris irure incididunt est ea occaecat sunt."
        type = ResultType.numberarray
        path = "/path/to/numbersfile"
        size = ResultSizeType.big
        format = "%f12%"
        data1 = DataResults(Description=desc, Help=help,
                            Type=type, Path=path, Size=size, Format=format)
        return data1

    def test_Create_Constructor_DataResults(self):
        desc = "Irure aute ex irure fugiat."
        help = "Ex consequat nostrud enim nulla dolore incididunt officia ullamco ut consequat mollit nulla."
        type = ResultType.image
        path = "/path/to/image"
        size = ResultSizeType.normal
        format = "f2%"
        data = self.get_DataResults1()

        self.assertEqual(desc, data.Description)
        self.assertEqual(help, data.Help)
        self.assertEqual(type, data.Type)
        self.assertEqual(path, data.Path)
        self.assertEqual(size, data.Size)
        self.assertEqual(format, data.Format)

    def test_DataResults_Not_Equals(self):
        data1 = self.get_DataResults1()
        data2 = self.get_DataResults2()
        self.assertNotEqual(data1, data2)

    def test_DataResults_GetJSON(self):
        data1 = self.get_DataResults1()
        dataR1json = data1.getJSON()
        self.assertTrue(len(dataR1json) > 0)
        self.assertTrue( dataR1json.index(data1.Description) > 0)

    def test_DataResults_LoadJSON(self):
        """Test Load DataResults from JSON
        """
        desc = "Irure aute ex irure fugiat."
        help = "Ex consequat nostrud enim nulla dolore incididunt officia ullamco ut consequat mollit nulla."
        type = ResultType.image
        path = "/path/to/image"
        size = ResultSizeType.normal
        format = "f2%"
        datajson='{"Description": "Irure aute ex irure fugiat.", "Help": "Ex consequat nostrud enim nulla dolore incididunt officia ullamco ut consequat mollit nulla.", "Type": "image", "Path": "/path/to/image", "Size": "normal", "Format": "f2%"}'
        data = DataResults.fromJSON(datajson)
        self.assertEqual(desc, data.Description)
        self.assertEqual(help, data.Help)
        self.assertEqual(type, data.Type)
        self.assertEqual(path, data.Path)
        self.assertEqual(size, data.Size)
        self.assertEqual(format, data.Format)




from enum import Enum
import json


def compare(s, t):
    """Compare two list without check the index

    Args:
        s (list): List A
        t (list): List B

    Returns:
        [boolean]: True if the two list have the same values.
    """
    t = list(t)   # make a mutable copy
    try:
        for elem in s:
            t.remove(elem)
    except ValueError:
        return False
    return not t


class DisplayType(str, Enum):
    """Display Enum Type
    """
    grid = "grid"
    """Display Type grid.
    """
    row = "row"
    """Display Type row.
    """

class ResultType(str, Enum):
    """Result Enum Type
    """
    str = "str"
    """Result Type string.
    """
    int = "int"
    """Result Type integer.
    """
    float = "float"
    """Result Type float.
    """
    percent = "percent"
    """Result Type percent without the % simbol.
    """
    image = "image"
    """Result Type image.
    """
    images = "images"
    """Result Type array of images.
    """
    numberarray = "numberarray"
    """Result Type array number.
    """

class ResultSizeType(str, Enum):
    """Result Size Type.
    """
    normal = "normal"
    """Result Type normal.
    """
    wide = "wide"
    """Result Type wide.
    """
    big = "big"
    """Result Type big.
    """


class ComplexEncoder(json.JSONEncoder):
    """Complex Encoder Json

    Args:
        json ([type]): Object to  convert to JSON
    """
    def default(self, obj):
        if hasattr(obj, '__toJSON'):
            return obj.__toJSON()
        else:
            return json.JSONEncoder.default(self, obj)


class DataResults(object):
    def __init__(self, Description, Help, Type, Path, Size, Format):
        self.Description = Description
        self.Help = Help
        self.Type = Type
        self.Path = Path
        self.Size = Size
        self.Format = Format

    def __eq__(self, obj):
        return self.Description == obj.Description and self.Help == obj.Help \
            and self.Type == obj.Type and self.Path == obj.Path \
            and self.Size == obj.Size and self.Format == obj.Format

    def __toJSON(obj):
        return dict(Description=obj.Description, Help=obj.Help, Type=obj.Type, Path=obj.Path, Size=obj.Size, Format=obj.Format)

    def getJSON(self):
        return json.dumps(DataResults.__toJSON(self), cls=ComplexEncoder)

    def __fromJSON(dct):
        return DataResults(dct["Description"], dct["Help"], dct["Type"], dct["Path"], dct["Size"], dct["Format"])

    def fromJSON(strjson):
        return json.loads(strjson, object_hook=DataResults.__fromJSON)


class DataLog(object):
    def __init__(self, Description, Path):
        self.Description = Description
        self.Path = Path

    def __eq__(self, obj):
        r_local = self.Path
        r_obj = obj.Path
        return self.Description == obj.Description and compare(r_local, r_obj)

    def __toJSON(obj):
        return dict(Description=obj.Description, Path=obj.Path)

    def getJSON(self):
        return json.dumps(DataLog.__toJSON(self), cls=ComplexEncoder)

    def __fromJSON(dct):
        return DataLog(dct["Description"], dct["Path"])

    def fromJSON(strjson):
        return json.loads(strjson, object_hook=DataLog.__fromJSON)


class DataExport(object):
    def __init__(self, Version, HasError, DisplayType, Results, Logs=DataLog("", [])):
        """DataExport

        Args:
            Version ([float]): Number Version of the Model.
            HasError ([number]): 0  for Error and 1 for Not error
            DisplayType ([DisplayType]): Display Type
            Results ([list of DataResults]): List of DataResults
        """
        self.Version = Version
        self.HasError = HasError
        self.DisplayType = DisplayType
        if Results is None:
            Results = []
        self.Results = Results
        if Logs is None:
            Logs = DataLog("", [])
        if Logs.Path is None:
            Logs.Path = []
        self.Logs = Logs

    def __eq__(self, obj):
        r_local = self.Results
        r_obj = obj.Results
        return self.Version == obj.Version and self.HasError == obj.HasError \
            and self.DisplayType == obj.DisplayType and compare(r_local, r_obj) and self.Logs == obj.Logs

    def __toJSON(obj):
        resultsorig = obj.Results
        results_json = list(map(lambda x: x.getJSON(), resultsorig))
        if obj.Logs is None:
            log = DataLog("", [])
            log_json = log.getJSON()
        else:
            log_json = obj.Logs.getJSON()
        return dict(Version=obj.Version, HasError=obj.HasError, DisplayType=obj.DisplayType, Results=results_json, Logs=log_json)

    def getJSON(self):
        return json.dumps(DataExport.__toJSON(self))

    def __fromJSON(dct):
        strarray = dct["Results"]
        results_json = list(map(lambda x: DataResults.fromJSON(x), strarray))
        if dct["Logs"] is None:
            logs = DataLog("", [])
        else:
            logs = DataLog.fromJSON(dct["Logs"])
        return DataExport(dct["Version"], dct["HasError"], dct["DisplayType"], results_json, logs)

    def fromJSON(strjson):
        return json.loads(strjson, object_hook=DataExport.__fromJSON)
